searchData={"content_type":"text/markdown","items":[{"doc":"`lastfm_archive` is a tool for extracting and archiving Last.fm music listening data - scrobbles.\n\nCurrent usage:\n- `sync/0`, `sync/2`: create and sync Lastfm scrobble data to local file archives\n- `transform/0`, `transform/2`: transform downloaded raw data in file archives into columnar and other formats, e.g. CSV, Apache Parquet, Arrow\n- `read/2`: load daily, monthly, yearly and entire dataset from file archives into data frames\n- `load_archive/2`: load all CSV data from the archive into Solr","ref":"LastfmArchive.html","title":"LastfmArchive","type":"module"},{"doc":"Returns the default coofigured Lastfm user","ref":"LastfmArchive.html#default_user/0","title":"LastfmArchive.default_user/0","type":"function"},{"doc":"Returns the total playcount and registered, i.e. earliest scrobble time for a user.","ref":"LastfmArchive.html#info/0","title":"LastfmArchive.info/0","type":"function"},{"doc":"Load all CSV data from the archive into Solr for a Lastfm user.\n\nThe function finds CSV files from the archive and sends them to\nSolr for ingestion one at a time. It uses `Hui` client to interact\nwith Solr and the `t:Hui.URL.t/0` struct\nfor Solr endpoint specification.\n\n#","ref":"LastfmArchive.html#load_archive/2","title":"LastfmArchive.load_archive/2","type":"function"},{"doc":"```\n  # define a Solr endpoint with %Hui.URL{} struct\n  headers = [{\"Content-type\", \"application/json\"}]\n  url = %Hui.URL{url: \"http://localhost:8983/solr/lastfm_archive\", handler: \"update\", headers: headers}\n\n  LastfmArchive.load_archive(\"a_lastfm_user\", url)\n```\n\nCSV files must be pre-created before the loading - see\n`transform/2`.","ref":"LastfmArchive.html#load_archive/2-example","title":"Example - LastfmArchive.load_archive/2","type":"function"},{"doc":"Read from an archive of a Lastfm user.\n\nThis returns scrobbles for a single day or month period\nin a lazy Explorer.DataFrame for further data manipulation\nand visualisation.\n\n#","ref":"LastfmArchive.html#read/2","title":"LastfmArchive.read/2","type":"function"},{"doc":"```\n  # read a single-day scrobbles from the configured\n  # archive (FileArchive) and default user\n  LastfmArchive.read(day: ~D[2022-12-31])\n\n  # read a single-month scrobbles for a user\n  LastfmArchive.read(\"a_lastfm_user\",  month: ~D[2022-12-31])\n```\n\nOptions:\n- `:day` - read scrobbles for this particular date (`Date.t()`)\n- `:month` - read scrobbles for this particular month (`Date.t()`)\n\nThis function can also return a lazy data frame from derived archive.\ni.e. CSV, Parquet archives created via `transform/2`.\n\n#","ref":"LastfmArchive.html#read/2-example","title":"Example - LastfmArchive.read/2","type":"function"},{"doc":"```\n  # read a single year of scrobbles for a user from Parquet archive\n  LastfmArchive.read(\"a_lastfm_user\", format: :parquet, year: 2023)\n\n  # read everything for a user from Parquet archive\n  LastfmArchive.read(\"a_lastfm_user\", format: :parquet)\n```\n\nOptions:\n- `:format` (required) - derived archive format: `:csv`, `:parquet`, `:ipc`, `:ipc_stream`\n- `:facet` - type of archive: `:scrobbles` (default), `:albums`, `:artists` or `:tracks`\n- `:year` - only read scrobbles for this particular year\n- `:columns` - an atom list for retrieving only a columns subset, available columns:\n`:album`, `:album_mbid`, `:artist`, `:artist_mbid`, `:artist_url`, `:date`, `:datetime`, `:datetime_unix`, `:id`, `:mbid`, `:mmdd`, `:name`, `:url`, `:year`","ref":"LastfmArchive.html#read/2-example","title":"Example - LastfmArchive.read/2","type":"function"},{"doc":"Sync scrobbles for a Lastfm user.\n\n#","ref":"LastfmArchive.html#sync/2","title":"LastfmArchive.sync/2","type":"function"},{"doc":"```\n  LastfmArchive.sync(\"a_lastfm_user\")\n```\n\nYou can also specify a default user is in configuration,\nfor example `user_a` in `config/config.exs`:\n\n```\n  config :lastfm_archive,\n    user: \"user_a\",\n    ... # other archiving options\n```\n\nAnd run:\n\n```\n  LastfmArchive.sync\n```\n\nThe first sync downloads all daily scrobbles in 200-track (gzip compressed)\nchunks that are written into a local file archive. Subsequent syncs extract\nfurther scrobbles starting from the date of latest downloaded scrobbles.\n\nThe data is currently in raw Lastfm `recenttracks` JSON format, chunked into\n200-track (max) `gzip` compressed pages and stored within directories corresponding\nto the days when tracks were scrobbled.\n\nOptions:\n\n- `:data_dir` - default `lastfm_data`, file archives are stored within this directory\n\n- `:date` - archive scrobbles from this date only\n\n- `:interval` - default `1000`(ms), the duration between successive Lastfm API requests.\nThis default interval ensures a safe request rate that is\nwithin Lastfm's term of service: no more than 5 requests per second\n\n- `:overwrite` - default `false`, if sets to true\nthe tool will (re)fetch and overwrite any previously downloaded\ndata. Use this option to refresh the file archive. Otherwise (false),\nthe system will not be making calls to Lastfm to re-fetch data\nif existing data chunks / pages are found.\n\n- `:per_page` - default `200`, number of scrobbles per file. The default is the\nmax number of tracks per request permissible by Lastfm\n\n- `:year` - archive scrobbles from this year only\n\n`:interval`, `:per_page` and `:data_dir` options can be configured in `config/config.exs`:\n\n```\n  config :lastfm_archive,\n    ...\n    data_dir: \"./lastfm_data/\"\n```","ref":"LastfmArchive.html#sync/2-example","title":"Example - LastfmArchive.sync/2","type":"function"},{"doc":"Transform downloaded file archive into CSV or Apache Parquet formats for a Lastfm user.\n\n#","ref":"LastfmArchive.html#transform/2","title":"LastfmArchive.transform/2","type":"function"},{"doc":"```\n  LastfmArchive.transform(\"a_lastfm_user\", format: :csv)\n\n  # transform archive of the default user into CSV files\n  LastfmArchive.transform()\n```\n\nThe function only transforms downloaded archive data on local filesystem. It does not fetch data from Lastfm,\nwhich can be done via `sync/2`.\n\nThe transformed files are created on a yearly basis and stored in `gzip` compressed format.\nThey are stored in a `csv` or `parquet` directory within either the default `./lastfm_data/`\nor the directory specified in config/config.exs (`:lastfm_archive, :data_dir`).\n\nOptions:\n- `:format` - format into which file archive is transformed: `:csv`, `:parquet`, `:ipc`, `:ipc_stream` (default)\n- `:facet` - type of archive: `:scrobbles` (default), `:albums`, `:artists` or `:tracks`\n- `:overwrite` existing data, default: false\n- `:year` - optionally transform data from this particular year","ref":"LastfmArchive.html#transform/2-example","title":"Example - LastfmArchive.transform/2","type":"function"},{"doc":"","ref":"LastfmArchive.html#t:metadata/0","title":"LastfmArchive.metadata/0","type":"type"},{"doc":"","ref":"LastfmArchive.html#t:options/0","title":"LastfmArchive.options/0","type":"type"},{"doc":"","ref":"LastfmArchive.html#t:solr_url/0","title":"LastfmArchive.solr_url/0","type":"type"},{"doc":"","ref":"LastfmArchive.html#t:time_range/0","title":"LastfmArchive.time_range/0","type":"type"},{"doc":"An archive derived from local data extracted from Lastfm.","ref":"LastfmArchive.Archive.DerivedArchive.html","title":"LastfmArchive.Archive.DerivedArchive","type":"module"},{"doc":"","ref":"LastfmArchive.Archive.DerivedArchive.html#t:read_options/0","title":"LastfmArchive.Archive.DerivedArchive.read_options/0","type":"type"},{"doc":"An archive containing raw data extracted from Lastfm API.","ref":"LastfmArchive.Archive.FileArchive.html","title":"LastfmArchive.Archive.FileArchive","type":"module"},{"doc":"","ref":"LastfmArchive.Archive.FileArchive.html#t:read_options/0","title":"LastfmArchive.Archive.FileArchive.read_options/0","type":"type"},{"doc":"Struct representing Lastfm archive metadata.","ref":"LastfmArchive.Archive.Metadata.html","title":"LastfmArchive.Archive.Metadata","type":"module"},{"doc":"Data struct containing new and some default metadata of an archive.\n\nOther metadata fields such as temporal, modified can be populated\nbased on the outcomes of archiving, i.e. the implementation of the\ncallbacks of this behaviour.","ref":"LastfmArchive.Archive.Metadata.html#new/1","title":"LastfmArchive.Archive.Metadata.new/1","type":"function"},{"doc":"","ref":"LastfmArchive.Archive.Metadata.html#new/2","title":"LastfmArchive.Archive.Metadata.new/2","type":"function"},{"doc":"","ref":"LastfmArchive.Archive.Metadata.html#new/3","title":"LastfmArchive.Archive.Metadata.new/3","type":"function"},{"doc":"","ref":"LastfmArchive.Archive.Metadata.html#t:facet_type/0","title":"LastfmArchive.Archive.Metadata.facet_type/0","type":"type"},{"doc":"Metadata descriping a Lastfm archive based on\n  [Dublin Core Metadata Initiative](https://www.dublincore.org/specifications/dublin-core/dcmi-terms/).","ref":"LastfmArchive.Archive.Metadata.html#t:t/0","title":"LastfmArchive.Archive.Metadata.t/0","type":"type"},{"doc":"Struct representing a Lastfm scrobble, i.e. listened track.","ref":"LastfmArchive.Archive.Scrobble.html","title":"LastfmArchive.Archive.Scrobble","type":"module"},{"doc":"Create a single or stream of scrobble structs.","ref":"LastfmArchive.Archive.Scrobble.html#new/1","title":"LastfmArchive.Archive.Scrobble.new/1","type":"function"},{"doc":"Lastfm scrobble","ref":"LastfmArchive.Archive.Scrobble.html#t:t/0","title":"LastfmArchive.Archive.Scrobble.t/0","type":"type"},{"doc":"Transform scrobbles into faceted columnar archive.\n\nThis transformer reads and transforms raw scrobbles data into\na faceted columnar archive, e.g. unique artists, albums, tracks","ref":"LastfmArchive.Archive.Transformers.FacetsTransformer.html","title":"LastfmArchive.Archive.Transformers.FacetsTransformer","type":"module"},{"doc":"Transform existing data of file archive into different storage formats.\n\nThis transformer simply reads and converts raw Scrobbles data into\ncolumnar and other storage formats. It does not apply further data transformation\nor data manipulation.","ref":"LastfmArchive.Archive.Transformers.LocalFileArchiveTransformer.html","title":"LastfmArchive.Archive.Transformers.LocalFileArchiveTransformer","type":"module"},{"doc":"Base transformer with default implementations.","ref":"LastfmArchive.Archive.Transformers.Transformer.html","title":"LastfmArchive.Archive.Transformers.Transformer","type":"module"},{"doc":"","ref":"LastfmArchive.Archive.Transformers.Transformer.html#apply/3","title":"LastfmArchive.Archive.Transformers.Transformer.apply/3","type":"function"},{"doc":"","ref":"LastfmArchive.Archive.Transformers.Transformer.html#data_frame_sink/4","title":"LastfmArchive.Archive.Transformers.Transformer.data_frame_sink/4","type":"function"},{"doc":"","ref":"LastfmArchive.Archive.Transformers.Transformer.html#data_frame_source/2","title":"LastfmArchive.Archive.Transformers.Transformer.data_frame_source/2","type":"function"},{"doc":"","ref":"LastfmArchive.Archive.Transformers.Transformer.html#facet_transformer_config/1","title":"LastfmArchive.Archive.Transformers.Transformer.facet_transformer_config/1","type":"function"},{"doc":"","ref":"LastfmArchive.Archive.Transformers.Transformer.html#facets/0","title":"LastfmArchive.Archive.Transformers.Transformer.facets/0","type":"function"},{"doc":"","ref":"LastfmArchive.Archive.Transformers.Transformer.html#format_config/1","title":"LastfmArchive.Archive.Transformers.Transformer.format_config/1","type":"function"},{"doc":"","ref":"LastfmArchive.Archive.Transformers.Transformer.html#formats/0","title":"LastfmArchive.Archive.Transformers.Transformer.formats/0","type":"function"},{"doc":"","ref":"LastfmArchive.Archive.Transformers.Transformer.html#mimetype/1","title":"LastfmArchive.Archive.Transformers.Transformer.mimetype/1","type":"function"},{"doc":"","ref":"LastfmArchive.Archive.Transformers.Transformer.html#read_opts/1","title":"LastfmArchive.Archive.Transformers.Transformer.read_opts/1","type":"function"},{"doc":"","ref":"LastfmArchive.Archive.Transformers.Transformer.html#transformer/1","title":"LastfmArchive.Archive.Transformers.Transformer.transformer/1","type":"function"},{"doc":"","ref":"LastfmArchive.Archive.Transformers.Transformer.html#validate_opts/1","title":"LastfmArchive.Archive.Transformers.Transformer.validate_opts/1","type":"function"},{"doc":"","ref":"LastfmArchive.Archive.Transformers.Transformer.html#write_opts/1","title":"LastfmArchive.Archive.Transformers.Transformer.write_opts/1","type":"function"},{"doc":"Behaviour of a Lastfm archive.\n\nAn archive contains transformed or scrobbles data retrieved from Lastfm API.\nIt can be based upon various storage implementation such as file systems and\ndatabases.","ref":"LastfmArchive.Behaviour.Archive.html","title":"LastfmArchive.Behaviour.Archive","type":"behaviour"},{"doc":"Archives all scrobbles data for a Lastfm user.\n\nOptional for post-hoc archives that are based on existing\nlocal archive such as CSV, Parquet archives.","ref":"LastfmArchive.Behaviour.Archive.html#c:archive/3","title":"LastfmArchive.Behaviour.Archive.archive/3","type":"callback"},{"doc":"Returns metadata of an existing archive.","ref":"LastfmArchive.Behaviour.Archive.html#c:describe/2","title":"LastfmArchive.Behaviour.Archive.describe/2","type":"callback"},{"doc":"Optionally applies post-archive side effects such as transformation into columnar storage.","ref":"LastfmArchive.Behaviour.Archive.html#c:post_archive/3","title":"LastfmArchive.Behaviour.Archive.post_archive/3","type":"callback"},{"doc":"Read access to the archive, returns an Explorer DataFrame for further data manipulation.","ref":"LastfmArchive.Behaviour.Archive.html#c:read/2","title":"LastfmArchive.Behaviour.Archive.read/2","type":"callback"},{"doc":"Writes latest metadata to file.","ref":"LastfmArchive.Behaviour.Archive.html#c:update_metadata/2","title":"LastfmArchive.Behaviour.Archive.update_metadata/2","type":"callback"},{"doc":"","ref":"LastfmArchive.Behaviour.Archive.html#t:api/0","title":"LastfmArchive.Behaviour.Archive.api/0","type":"type"},{"doc":"","ref":"LastfmArchive.Behaviour.Archive.html#t:metadata/0","title":"LastfmArchive.Behaviour.Archive.metadata/0","type":"type"},{"doc":"","ref":"LastfmArchive.Behaviour.Archive.html#t:options/0","title":"LastfmArchive.Behaviour.Archive.options/0","type":"type"},{"doc":"","ref":"LastfmArchive.Behaviour.Archive.html#t:scrobbles/0","title":"LastfmArchive.Behaviour.Archive.scrobbles/0","type":"type"},{"doc":"","ref":"LastfmArchive.Behaviour.Archive.html#t:transformer/0","title":"LastfmArchive.Behaviour.Archive.transformer/0","type":"type"},{"doc":"","ref":"LastfmArchive.Behaviour.Archive.html#t:user/0","title":"LastfmArchive.Behaviour.Archive.user/0","type":"type"},{"doc":"Behaviour, macro and functions for Explorer.DataFrame related I/Os.","ref":"LastfmArchive.Behaviour.DataFrameIo.html","title":"LastfmArchive.Behaviour.DataFrameIo","type":"behaviour"},{"doc":"","ref":"LastfmArchive.Behaviour.DataFrameIo.html#c:dump_csv!/2","title":"LastfmArchive.Behaviour.DataFrameIo.dump_csv!/2","type":"callback"},{"doc":"","ref":"LastfmArchive.Behaviour.DataFrameIo.html#c:dump_ipc!/2","title":"LastfmArchive.Behaviour.DataFrameIo.dump_ipc!/2","type":"callback"},{"doc":"","ref":"LastfmArchive.Behaviour.DataFrameIo.html#c:dump_ipc_stream!/2","title":"LastfmArchive.Behaviour.DataFrameIo.dump_ipc_stream!/2","type":"callback"},{"doc":"","ref":"LastfmArchive.Behaviour.DataFrameIo.html#c:dump_parquet!/2","title":"LastfmArchive.Behaviour.DataFrameIo.dump_parquet!/2","type":"callback"},{"doc":"","ref":"LastfmArchive.Behaviour.DataFrameIo.html#c:from_csv!/2","title":"LastfmArchive.Behaviour.DataFrameIo.from_csv!/2","type":"callback"},{"doc":"","ref":"LastfmArchive.Behaviour.DataFrameIo.html#c:from_ipc!/2","title":"LastfmArchive.Behaviour.DataFrameIo.from_ipc!/2","type":"callback"},{"doc":"","ref":"LastfmArchive.Behaviour.DataFrameIo.html#c:from_ipc_stream!/2","title":"LastfmArchive.Behaviour.DataFrameIo.from_ipc_stream!/2","type":"callback"},{"doc":"","ref":"LastfmArchive.Behaviour.DataFrameIo.html#c:from_parquet!/2","title":"LastfmArchive.Behaviour.DataFrameIo.from_parquet!/2","type":"callback"},{"doc":"","ref":"LastfmArchive.Behaviour.DataFrameIo.html#c:to_csv!/3","title":"LastfmArchive.Behaviour.DataFrameIo.to_csv!/3","type":"callback"},{"doc":"","ref":"LastfmArchive.Behaviour.DataFrameIo.html#c:to_ipc!/3","title":"LastfmArchive.Behaviour.DataFrameIo.to_ipc!/3","type":"callback"},{"doc":"","ref":"LastfmArchive.Behaviour.DataFrameIo.html#c:to_ipc_stream!/3","title":"LastfmArchive.Behaviour.DataFrameIo.to_ipc_stream!/3","type":"callback"},{"doc":"","ref":"LastfmArchive.Behaviour.DataFrameIo.html#c:to_parquet!/3","title":"LastfmArchive.Behaviour.DataFrameIo.to_parquet!/3","type":"callback"},{"doc":"","ref":"LastfmArchive.Behaviour.DataFrameIo.html#t:data/0","title":"LastfmArchive.Behaviour.DataFrameIo.data/0","type":"type"},{"doc":"","ref":"LastfmArchive.Behaviour.DataFrameIo.html#t:data_frame/0","title":"LastfmArchive.Behaviour.DataFrameIo.data_frame/0","type":"type"},{"doc":"","ref":"LastfmArchive.Behaviour.DataFrameIo.html#t:filepath/0","title":"LastfmArchive.Behaviour.DataFrameIo.filepath/0","type":"type"},{"doc":"","ref":"LastfmArchive.Behaviour.DataFrameIo.html#t:options/0","title":"LastfmArchive.Behaviour.DataFrameIo.options/0","type":"type"},{"doc":"Behaviour and data struct for retrieving data from Lastfm API.","ref":"LastfmArchive.Behaviour.LastfmClient.html","title":"LastfmArchive.Behaviour.LastfmClient","type":"behaviour"},{"doc":"Returns the total playcount, registered, i.e. first scrobble time for a user.","ref":"LastfmArchive.Behaviour.LastfmClient.html#c:info/2","title":"LastfmArchive.Behaviour.LastfmClient.info/2","type":"callback"},{"doc":"Returns the playcount and the latest scrobble date of a user for a given time range.","ref":"LastfmArchive.Behaviour.LastfmClient.html#c:playcount/3","title":"LastfmArchive.Behaviour.LastfmClient.playcount/3","type":"callback"},{"doc":"Returns the scrobbles of a user for a given time range.\n\nSee Lastfm API [documentation](https://www.last.fm/api/show/user.getRecentTracks) for more details.","ref":"LastfmArchive.Behaviour.LastfmClient.html#c:scrobbles/3","title":"LastfmArchive.Behaviour.LastfmClient.scrobbles/3","type":"callback"},{"doc":"","ref":"LastfmArchive.Behaviour.LastfmClient.html#t:from/0","title":"LastfmArchive.Behaviour.LastfmClient.from/0","type":"type"},{"doc":"","ref":"LastfmArchive.Behaviour.LastfmClient.html#t:lastfm_api/0","title":"LastfmArchive.Behaviour.LastfmClient.lastfm_api/0","type":"type"},{"doc":"","ref":"LastfmArchive.Behaviour.LastfmClient.html#t:latest_scrobble_time/0","title":"LastfmArchive.Behaviour.LastfmClient.latest_scrobble_time/0","type":"type"},{"doc":"","ref":"LastfmArchive.Behaviour.LastfmClient.html#t:limit/0","title":"LastfmArchive.Behaviour.LastfmClient.limit/0","type":"type"},{"doc":"","ref":"LastfmArchive.Behaviour.LastfmClient.html#t:page/0","title":"LastfmArchive.Behaviour.LastfmClient.page/0","type":"type"},{"doc":"","ref":"LastfmArchive.Behaviour.LastfmClient.html#t:playcount/0","title":"LastfmArchive.Behaviour.LastfmClient.playcount/0","type":"type"},{"doc":"","ref":"LastfmArchive.Behaviour.LastfmClient.html#t:registered_time/0","title":"LastfmArchive.Behaviour.LastfmClient.registered_time/0","type":"type"},{"doc":"","ref":"LastfmArchive.Behaviour.LastfmClient.html#t:to/0","title":"LastfmArchive.Behaviour.LastfmClient.to/0","type":"type"},{"doc":"","ref":"LastfmArchive.Behaviour.LastfmClient.html#t:user/0","title":"LastfmArchive.Behaviour.LastfmClient.user/0","type":"type"},{"doc":"Transform strategy for applying post archive side affects","ref":"LastfmArchive.Behaviour.Transformer.html","title":"LastfmArchive.Behaviour.Transformer","type":"behaviour"},{"doc":"","ref":"LastfmArchive.Behaviour.Transformer.html#c:sink/3","title":"LastfmArchive.Behaviour.Transformer.sink/3","type":"callback"},{"doc":"","ref":"LastfmArchive.Behaviour.Transformer.html#c:source/2","title":"LastfmArchive.Behaviour.Transformer.source/2","type":"callback"},{"doc":"","ref":"LastfmArchive.Behaviour.Transformer.html#c:transform/2","title":"LastfmArchive.Behaviour.Transformer.transform/2","type":"callback"},{"doc":"","ref":"LastfmArchive.Behaviour.Transformer.html#t:data_frame/0","title":"LastfmArchive.Behaviour.Transformer.data_frame/0","type":"type"},{"doc":"","ref":"LastfmArchive.Behaviour.Transformer.html#t:metadata/0","title":"LastfmArchive.Behaviour.Transformer.metadata/0","type":"type"},{"doc":"","ref":"LastfmArchive.Behaviour.Transformer.html#t:options/0","title":"LastfmArchive.Behaviour.Transformer.options/0","type":"type"},{"doc":"Client for extracting Lastfm user info and scrobbles data via the official API.","ref":"LastfmArchive.LastfmClient.Impl.html","title":"LastfmArchive.LastfmClient.Impl","type":"module"},{"doc":"","ref":"LastfmArchive.LastfmClient.Impl.html#default_user/0","title":"LastfmArchive.LastfmClient.Impl.default_user/0","type":"function"},{"doc":"Returns the total playcount and earliest scrobble date for a user.","ref":"LastfmArchive.LastfmClient.Impl.html#info/2","title":"LastfmArchive.LastfmClient.Impl.info/2","type":"function"},{"doc":"Returns the playcount of a user for a given time range.","ref":"LastfmArchive.LastfmClient.Impl.html#playcount/3","title":"LastfmArchive.LastfmClient.Impl.playcount/3","type":"function"},{"doc":"Returns the scrobbles of a user for a given time range.\n\nSee Lastfm API [documentation](https://www.last.fm/api/show/user.getRecentTracks) for more details.","ref":"LastfmArchive.LastfmClient.Impl.html#scrobbles/3","title":"LastfmArchive.LastfmClient.Impl.scrobbles/3","type":"function"},{"doc":"Struct representing Lastfm API.","ref":"LastfmArchive.LastfmClient.LastfmApi.html","title":"LastfmArchive.LastfmClient.LastfmApi","type":"module"},{"doc":"","ref":"LastfmArchive.LastfmClient.LastfmApi.html#new/1","title":"LastfmArchive.LastfmClient.LastfmApi.new/1","type":"function"},{"doc":"Lastfm API","ref":"LastfmArchive.LastfmClient.LastfmApi.html#t:t/0","title":"LastfmArchive.LastfmClient.LastfmApi.t/0","type":"type"},{"doc":"Livebook chart and text rendering.","ref":"LastfmArchive.Livebook.html","title":"LastfmArchive.Livebook","type":"module"},{"doc":"Returns a list of daily scrobble playcounts and stats per years.","ref":"LastfmArchive.Livebook.html#daily_playcounts_per_years/2","title":"LastfmArchive.Livebook.daily_playcounts_per_years/2","type":"function"},{"doc":"Display user name and total number of scrobbles to archive.","ref":"LastfmArchive.Livebook.html#info/1","title":"LastfmArchive.Livebook.info/1","type":"function"},{"doc":"Display faceted dataframe in VegaLite bubble plot showing first play and counts (size, colour).","ref":"LastfmArchive.Livebook.html#render_first_play_bubble_plot/1","title":"LastfmArchive.Livebook.render_first_play_bubble_plot/1","type":"function"},{"doc":"Display daily playcounts of scrobbles archived in VegaLite heatmaps.","ref":"LastfmArchive.Livebook.html#render_playcounts_heatmaps/3","title":"LastfmArchive.Livebook.render_playcounts_heatmaps/3","type":"function"},{"doc":"","ref":"LastfmArchive.Livebook.html#t:daily_playcounts/0","title":"LastfmArchive.Livebook.daily_playcounts/0","type":"type"},{"doc":"","ref":"LastfmArchive.Livebook.html#t:user/0","title":"LastfmArchive.Livebook.user/0","type":"type"},{"doc":"","ref":"LastfmArchive.Livebook.html#t:year/0","title":"LastfmArchive.Livebook.year/0","type":"type"},{"doc":"This module provides functions for loading Lastfm data into databases and search engines.","ref":"LastfmArchive.Load.html","title":"LastfmArchive.Load","type":"module"},{"doc":"Check a Solr core/collection to ensure it has the required Lastfm data fields.\n\nThe check currently inspects Solr schema for a list of Lastfm fields\nand returns error if one or more of the fields are missing.\nSee `LastfmArchive.Transform.transform/3` for the list of fields.\n\n#","ref":"LastfmArchive.Load.html#check_solr_schema/1","title":"LastfmArchive.Load.check_solr_schema/1","type":"function"},{"doc":"```\n  LastfmArchive.Load.check_solr_schema(\"http://solr_url...\")\n  LastfmArchive.Load.check_solr_schema(:lastfm_archive) # ping a configured endpoint\n```\n\nSee `ping_solr/1` for more details on URL configuration.","ref":"LastfmArchive.Load.html#check_solr_schema/1-example","title":"Example - LastfmArchive.Load.check_solr_schema/1","type":"function"},{"doc":"Load a CSV file data from the archive into Solr for a Lastfm user.\n\nThe function reads and converts scrobbles in a CSV file from the file\narchive into a list of maps. The maps are sent to Solr for ingestion.\nUse `t:Hui.URL.t/0` struct to specify the Solr endpoint.\n\n#","ref":"LastfmArchive.Load.html#load_solr/3","title":"LastfmArchive.Load.load_solr/3","type":"function"},{"doc":"```\n  # define a Solr endpoint with %Hui.URL{} struct\n  headers = [{\"Content-type\", \"application/json\"}]\n  url = %Hui.URL{url: \"http://localhost:8983/solr/lastfm_archive\", handler: \"update\", headers: headers}\n\n  # ingest data scrobbled in 2018\n  LastfmArchive.Load.load_solr(url, \"a_lastfm_user\", \"csv/2018.csv.gz\")\n```\n\nCSV files must be pre-created by transforming raw JSON Lastfm data - see\n`LastfmArchive.transform/2`.","ref":"LastfmArchive.Load.html#load_solr/3-example","title":"Example - LastfmArchive.Load.load_solr/3","type":"function"},{"doc":"Ping a Solr core/collection endpoint to check if it is running.\n\nThe endpoint can either be a URL string or an atom referring to an endpoint in configuration.\nThe library uses `Hui` to interact with Solr, an endpoint can be specified as below:\n\n#","ref":"LastfmArchive.Load.html#ping_solr/1","title":"LastfmArchive.Load.ping_solr/1","type":"function"},{"doc":"```\n  LastfmArchive.Load.ping_solr(\"http://solr_url...\")\n  LastfmArchive.Load.ping_solr(:lastfm_archive) # check a configured endpoint\n```\n\n`:lastfm_archive` refers to the following Solr update endpoint in configuration:\n\n```\n  config :hui, :lastfm_archive,\n    url: \"http://solr_url..\",\n    handler: \"update\",\n    headers: [{\"Content-type\", \"application/json\"}]\n```\n\nSee `Hui.URL` module for more details.","ref":"LastfmArchive.Load.html#ping_solr/1-example","title":"Example - LastfmArchive.Load.ping_solr/1","type":"function"},{"doc":"# Lastfm Archive [![Build Status](https://github.com/boonious/lastfm_archive/actions/workflows/elixir.yml/badge.svg)](https://github.com/boonious/lastfm_archive/actions/workflows/elixir.yml) [![Hex pm](http://img.shields.io/hexpm/v/lastfm_archive.svg?style=flat)](https://hex.pm/packages/lastfm_archive) [![Coverage Status](https://coveralls.io/repos/github/boonious/lastfm_archive/badge.svg)](https://coveralls.io/github/boonious/lastfm_archive?branch=master)\n\nA tool for extracting and archiving Last.fm music listening data - [scrobbles](https://www.last.fm/about/trackmymusic).\n\n**Note**: \n- check out: the new [Facets archiving](#facets-archiving) Livebook\n- previous analytics features have been migrated to [coda](https://github.com/boonious/coda), including [on this day](https://github.com/boonious/coda#livebook) Livebook.\nVisit [coda](https://github.com/boonious/coda) for future Lastfm analytics.","ref":"readme.html","title":"Lastfm Archive","type":"extras"},{"doc":"Download and create a file archive of Lastfm scrobble tracks via an [Elixir](https://elixir-lang.org)\napplication or [interactive Elixir](https://elixir-lang.org/getting-started/introduction.html#interactive-mode)\nby invoking `iex -S mix` command line action while in software home directory. \n \n```elixir\n  # archive all data of a default user specified in configuration\n  LastfmArchive.sync # subsequent calls download only latest scrobbles\n\n  # archive all data of any Lastfm user\n  # the data is stored in directory named after the user\n  LastfmArchive.sync(\"a_lastfm_user\")\n```\n\nYou can also deploy and use the tool in [Livebook](https://livebook.dev),\nas shown in various [Livebook guides](#livebook-guides).\n\nScrobbles are downloaded via the Last.fm API and stored in the file archive on demand\nand on a daily basis. The software has a built-in cache to remember \nand resume from the previous downloads. It skips already downloaded scrobbles and\ndoes not make further requests to the API.\n\nThe stored data is in a raw Last.fm `recenttracks` JSON format,\nchunked into 200-track (max) `gzip` compressed pages and stored within directories\ncorresponding to the days when tracks were scrobbled. The file archive in a main \ndirectory specified in configuration - see below.\n\nSee [Creating a file archive](#creating-a-file-archive) guide and [`sync/2`](https://hexdocs.pm/lastfm_archive/LastfmArchive.html#sync/2) for various archiving options such `overwrite`, `year`, `date`.\n\n#","ref":"readme.html#usage","title":"Usage - Lastfm Archive","type":"extras"},{"doc":"You can transform the file archive into other common storage formats such as CSV and \ncolumnar data structure such as [Apache Parquet](https://parquet.apache.org). \nThese formats facilitate data interoperability, as well as \n[OLAP](https://www.snowflake.com/guides/olap-vs-oltp), analytics use cases.\n\n```elixir\n# transform the file archive into columnar Apache Parquet files\nLastfmArchive.transform(\"a_lastfm_user\", format: :parquet)\n\n# to columnar Apache Arrow IPC files\nLastfmArchive.transform(\"a_lastfm_user\", format: :ipc_stream)\n\n# CSV format also available\nLastfmArchive.transform(\"a_lastfm_user\", format: :csv)\n```\n\nAvailable formats: \n- CSV (tab-delimited)\n- [Apache Arrow](https://arrow.apache.org) columnar format\n- [Apache Parquet](https://parquet.apache.org) columnar format\n\nSee [Columnar data transforms](#columnar-data-transforms) guide and \n[`transform/2`](https://hexdocs.pm/lastfm_archive/LastfmArchive.html#transform/2).\n\n#","ref":"readme.html#transform-into-columnar-storage-formats","title":"Transform into columnar storage formats - Lastfm Archive","type":"extras"},{"doc":"You can also transform the file archive into faceted (`artists`, `albums`, `tracks`)\ndatasets.\n\n```elixir\nLastfmArchive.transform(\"a_lastfm_user\", format: :ipc_stream, facet: :artists)\n```\n\nSee [Facets archiving](#facets-archiving) guide and \n[`transform/2`](https://hexdocs.pm/lastfm_archive/LastfmArchive.html#transform/2).\n\n#","ref":"readme.html#transform-into-faceted-columnar-datasets","title":"Transform into faceted columnar datasets - Lastfm Archive","type":"extras"},{"doc":"The tool provides a [`read/2`](https://hexdocs.pm/lastfm_archive/LastfmArchive.html#read/2)\nfunction for retrieving data from the archive. It mainly relies on\n[Elixir Explorer](https://github.com/elixir-explorer/explorer) data frame mechanisms\nto underpin further data i/o, manipulation, analytics and visualisation.\n\nThe function returns a lazy `t:Explorer.DataFrame.t/0`.\n\n##","ref":"readme.html#read-archive","title":"Read archive - Lastfm Archive","type":"extras"},{"doc":"Scrobbles stored in the file archive can be read \nwith a `day` or `month` option:\n\n```elixir\n# read a single-day scrobbles for the configured default user\nLastfmArchive.read(day: ~D[2022-12-31])\n\n# read a single-month scrobbles for a user with an arbitrary day of a month\nLastfmArchive.read(\"a_lastfm_user\",  month: ~D[2022-12-01])\n```\n\n##","ref":"readme.html#from-raw-data-file-archive","title":"From raw data file archive - Lastfm Archive","type":"extras"},{"doc":"[`read/2`](https://hexdocs.pm/lastfm_archive/LastfmArchive.html#read/2) can\nreturn a single-year or all scrobbles, i.e. **the entire dataset** from a columnar archive.\nA `columns` option is available to retrieve only a column subset.\n\n```elixir\n# load all 2023 data from a Parquet archive\nLastfmArchive.read(\"a_lastfm_user\", format: :parquet, year: 2023)\n\n# load all data from an Arrow IPC archive\nLastfmArchive.read(\"a_lastfm_user\", format: :ipc_stream)\n\n# load data from specific columns\nLastfmArchive.read(\"a_lastfm_user\", format: :parquet, columns: [:id, :artist, :album])\n```\n\n##","ref":"readme.html#from-columnar-archive-for-analytics","title":"From columnar archive for analytics - Lastfm Archive","type":"extras"},{"doc":"[`read/2`](https://hexdocs.pm/lastfm_archive/LastfmArchive.html#read/2) can also\nreturn the faceted datasets, e.g. **all artists** from a columnar archive.\n\n```elixir\nLastfmArchive.read(\"a_lastfm_user\", format: :ipc_stream, facet: :artists)\n```","ref":"readme.html#from-faceted-datasets-for-analytics","title":"From faceted datasets for analytics - Lastfm Archive","type":"extras"},{"doc":"`LastfmArchive` also provides the following [Livebook](https://livebook.dev) interactive and step-by-step guides.\n\n#","ref":"readme.html#livebook-guides","title":"Livebook guides - Lastfm Archive","type":"extras"},{"doc":"[![Run in Livebook](https://livebook.dev/badge/v1/blue.svg)](https://livebook.dev/run?url=https%3A%2F%2Fraw.githubusercontent.com%2Fboonious%2Flastfm_archive%2Fmaster%2Flivebook%2Farchiving.livemd)\n\n[Creating a file archive](https://hexdocs.pm/lastfm_archive/archiving.html) guide for creating a local file archive consisting data fetched from the Last.fm API. It provides a heatmap and count visualisation for checking ongoing archiving status.\n\n![archiving progress visualisation](assets/img/livebook_heatmap.png)\n\n#","ref":"readme.html#creating-a-file-archive","title":"Creating a file archive - Lastfm Archive","type":"extras"},{"doc":"[![Run in Livebook](https://livebook.dev/badge/v1/blue.svg)](https://livebook.dev/run?url=https%3A%2F%2Fraw.githubusercontent.com%2Fboonious%2Flastfm_archive%2Fmaster%2Flivebook%2Ftransforming.livemd)\n\n[Columnar data transforms](https://hexdocs.pm/lastfm_archive/transforming.html) guide for transforming the local file archive to columnar data formats (Arrow, Parquet). It demonstrates how `read/2` can be used to load single-year single-column data, as well as an entire dataset into data frame for various analytics.\n\nSee a [sample output](https://hexdocs.pm/lastfm_archive/assets/img/livebook_unique_tracks_analytics.png) of this guide,\nshowing top tracks analytics.\n\n#","ref":"readme.html#columnar-data-transforms","title":"Columnar data transforms - Lastfm Archive","type":"extras"},{"doc":"[![Run in Livebook](https://livebook.dev/badge/v1/blue.svg)](https://livebook.dev/run?url=https%3A%2F%2Fraw.githubusercontent.com%2Fboonious%2Flastfm_archive%2Fmaster%2Flivebook%2Ffacets.livemd)\n\n[Facets archiving](https://hexdocs.pm/lastfm_archive/facets.html) guide shows how the local file archive can generate faceted `artists`, `albums`, `tracks` columnar datasets. It also demos how the datasets may be used. For example finding the new artists discovered on a particular date,\n\n![new artists discovered on this day](assets/img/livebook_new_artists_on_this_day.png)\n\nand visualising all artists, when their were first listened to and overall popularity.\n\n![all artists first played and popularity](assets/img/livebook_firstplay_bubble_plot.png)","ref":"readme.html#facets-archiving","title":"Facets archiving - Lastfm Archive","type":"extras"},{"doc":"To load all transformed CSV data from the archive into Solr:\n\n\n```elixir\n  # define a Solr endpoint with %Hui.URL{} struct\n  headers = [{\"Content-type\", \"application/json\"}]\n  url = %Hui.URL{url: \"http://localhost:8983/solr/lastfm_archive\", handler: \"update\", headers: headers}\n\n  LastfmArchive.load_archive(\"a_lastfm_user\", url)\n```\n\nThe function finds CSV files from the archive and send them to\nSolr for ingestion one at a time. It uses `Hui` client to interact\nwith Solr and the `t:Hui.URL.t/0` struct for Solr endpoint specification.","ref":"readme.html#other-usage","title":"Other usage - Lastfm Archive","type":"extras"},{"doc":"This tool requires Elixir and Erlang, see [installation](https://elixir-lang.org/install.html) details\nfor various operating systems or [Livebook](https://livebook.dev).","ref":"readme.html#requirement","title":"Requirement - Lastfm Archive","type":"extras"},{"doc":"`lastfm_archive` is [available in Hex](https://hex.pm/packages/lastfm_archive),\nthe package can be installed by adding `lastfm_archive`\nto your list of dependencies in `mix.exs`:\n\n```elixir\n  def deps do\n    [\n      {:lastfm_archive, \"~> 1.2\"}\n    ]\n  end\n```\n\nDocumentation can be found at [https://hexdocs.pm/lastfm_archive](https://hexdocs.pm/lastfm_archive).","ref":"readme.html#installation","title":"Installation - Lastfm Archive","type":"extras"},{"doc":"Add the following entries in your config - `config/config.exs`. For example,\nthe following specifies an Lastfm `user` and a main file location for\nmultiple user archives, `./lastfm_data/` relative to the software home directory.\n\nYou also need to specify an `lastfm_api_key` in the config, so that the application can\n[access Lastfm API](https://www.last.fm/api/authentication).\n\n```elixir\n  config :lastfm_archive,\n    user: \"default_user\", # the default user\n    data_dir: \"./lastfm_data/\", # main directory for multiple archives\n    lastfm_api_key: \"api_key_provided_by_lastfm\",\n    per_page: 200, # 200 is max no. of tracks per call permitted by Lastfm API \n    interval: 1000 # milliseconds between requests cf. Lastfm's max 5 reqs/s rate limit\n\n\n  # optional: Solr endpoint for Lastfm data loading\n  config :hui, :lastfm_archive,\n    url: \"http://localhost:8983/solr/lastfm_archive\",\n    handler: \"update\",\n    headers: [{\"Content-type\", \"application/json\"}]\n\n```\n\nSee [`sync/2`](https://hexdocs.pm/lastfm_archive/LastfmArchive.html#sync/2)\nfor other configurable archiving options, e.g. `interval`, `per_page`.\n\nSee [`Hui`](https://hexdocs.pm/hui/readme.html#content) for more details on Solr configuration.\n\nAn `api_key` must be configured to enable Lastfm API requests,\nsee [https://www.last.fm/api](https://www.last.fm/api) (\"Get an API account\").","ref":"readme.html#configuration","title":"Configuration - Lastfm Archive","type":"extras"},{"doc":"# Changelog","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"* `FacetsTransformer` and `facet` (unique artists, albums, tracks) option in `transform/2` for facets archiving\n* refactor: utils modules, dataframe and transformer config macros\n* Livebook: \n    * [`Creating a file archive`](https://hexdocs.pm/lastfm_archive/archiving.html): heatmaps visualisation containing daily playcounts, tooltips and stats\n    * [`Facets archiving`](https://hexdocs.pm/lastfm_archive/facets.html) new guide about facets archiving with usage demo for `Explorer.DataFrame` analytics and [Vega-Lite](https://vega.github.io/vega-lite/) visualisation","ref":"changelog.html#1-2-0-2023-09-27","title":"1.2.0 (2023-09-27) - Changelog","type":"extras"},{"doc":"* fix: Lastfm client bug that fetches page 1 instead of page 2 (for scrobbles > 200 per-day/on the day)\n* test: use `ExMachina` test data factories, update tests and replace static test fixtures","ref":"changelog.html#1-1-1-2023-09-15","title":"1.1.1 (2023-09-15) - Changelog","type":"extras"},{"doc":"* `year`, `date`, `overwrite` options for `LastfmArchive.sync/2`\n* refactor: `.cache` hidden directory hosting cache files\n* refactor: split `LastfmArchive.Cache` into behaviour, client, server modules\n* refactor: extract date time logic from `LastfmArchive.Utils` into a separate module","ref":"changelog.html#1-1-0-2023-09-06","title":"1.1.0 (2023-09-06) - Changelog","type":"extras"},{"doc":"* faceted archive: deriving multiple facets (archives) from the same file archive in support of analytics use cases\n* transforms data on a year-by-year basis by default, instead of loading the entire dataset\n* refactor: `.metadata` and `derived` directories hosting various metadata files and derived archives\n* analytics features migrated to [coda](https://github.com/boonious/coda)","ref":"changelog.html#1-0-0-2023-09-01","title":"1.0.0 (2023-09-01) - Changelog","type":"extras"},{"doc":"* most-frequent, play-once samples analytics for on-this-day Livebook page\n* fine-grained `Transformer` behaviour with default implementation and base transformer\n* `DateTime`, `Date` typed columns (instead of string) in transformed columnar archive \n* refactor: rename `name` column (of songs) in columnar storage to `track`","ref":"changelog.html#0-11-1-2023-08-21","title":"0.11.1 (2023-08-21) - Changelog","type":"extras"},{"doc":"* new `Analytics` (generated callbacks) and `LivebookAnalytics` behaviours, with default implementations to separate the concerns of data frame analytics and Livebook rendering\n* analyics improvement:\n    * includes various artists albums\n    * more overall stats\n    * stats per facet, e.g. number of albums, tracks per artist\n* resolve on-this-day analytics ranking issue\n* fix issue that caused subsequent columnar data transforms to stuck at previous (first) date of transform","ref":"changelog.html#0-11-0-2023-08-11","title":"0.11.0 (2023-08-11) - Changelog","type":"extras"},{"doc":"* new Livebook to present analytics of all music played \"on this day\"","ref":"changelog.html#0-10-4-2023-08-03","title":"0.10.4 (2023-08-03) - Changelog","type":"extras"},{"doc":"* additional data storage formats (Apache Arrow IPC) via `Explorer.DataFrame` i/o functions\n* `read/2` all scrobbles, i.e. entire dataset into a lazy data frame by default\n* update and create new Livebook guides for archiving and columnar data transformation","ref":"changelog.html#0-10-3-2023-07-26","title":"0.10.3 (2023-07-26) - Changelog","type":"extras"},{"doc":"* `overwrite` option for file archive transformer and `transform/2`\n* `read/2` CSV and Parquet data (deprecate `read_csv`, `read_parquet`)\n* `columns` option to load only required CSV, Parquet columns into data frames \n* use Explorer DataFrame I/O functions to compress Parquet data\n* refactor: DataFrame I/O macro, generate tests for transform formats","ref":"changelog.html#0-10-2-2023-07-18","title":"0.10.2 (2023-07-18) - Changelog","type":"extras"},{"doc":"* `read/2` callback and implementation to return data in `Explorer` data frame\n* `after_archive` callback and implementation to transform data into TSV and Apache Parquet files\n* refactor `Archive`, `FileArchive` and new `Scrobble` and `Metadata` structs","ref":"changelog.html#0-10-1-2023-07-07","title":"0.10.1 (2023-07-07) - Changelog","type":"extras"},{"doc":"* first interactive notebook - Livebook support\n* provide an archiving Livebook to initiate archiving jobs\n* playcounts heatmap and table for visualising archiving progress","ref":"changelog.html#0-10-0-2023-06-07","title":"0.10.0 (2023-06-07) - Changelog","type":"extras"},{"doc":"* refactor: FileArchive - raw Lastfm scrobbles JSON format and better logging\n* default runtime configuration to better support Livebook usage","ref":"changelog.html#0-9-3-2023-06-02","title":"0.9.3 (2023-06-02) - Changelog","type":"extras"},{"doc":"* `info/0` returns total play count and first date of scrobble for default user\n* update Lastfm client to use Livebook system env vars (`LB_LFM_USER` and `LB_LFM_API_KEY`) along with config values \n* rename modules and tests in idiomatic namespaces\n* bump dependencies, Elixir / Erlang versions\n* Github Actions CI","ref":"changelog.html#0-9-2-2023-05-24","title":"0.9.2 (2023-05-24) - Changelog","type":"extras"},{"doc":"* `sync/2` handles request errors from Last.fm API\n* caches errors during sync so that subsequent syncs or API requests can be made to archive the missing scrobbles\n* do not cache sync results of today's scrobbles as it's a always partial sync","ref":"changelog.html#0-9-1-2021-05-05","title":"0.9.1 (2021-05-05) - Changelog","type":"extras"},{"doc":"* Create an `Archive` behaviour with `FileArchive` implementation\n* Extract and store scrobbles in daily chunks\n* Extensive refactoring: simpler LastfmArchive functions core, deprecate `archive` functions\n* Rewrite tests with mocks, eliminate file writes during tests\n* Implement a GenServer simple tick-based (auto) cache memoization to prevent repetitive API calls to Last.fm","ref":"changelog.html#0-9-0-2021-04-14","title":"0.9.0 (2021-04-14) - Changelog","type":"extras"},{"doc":"* Update Elixir/Erlang versions and dependencies\n* Replace HTTP client (HTTPoison) with Erlang's built-in httpc client\n* Replace Poison with Jason for JSON decoding/encoding\n* Apply mix formatting to the entire codebase\n* Refactor and abstract LastfmArchive.Extract functions into a new module based on `Lastfm.Client` behaviour\n* Implement explicit behaviour and contract-based unit testing (Hammox/Mox)\n* Refactor and enable concurrent testing","ref":"changelog.html#0-8-0-2021-01-04","title":"0.8.0 (2021-01-04) - Changelog","type":"extras"},{"doc":"* Fix the issue of Last.fm changing the date/count data type in JSON back and forth (string   integer).","ref":"changelog.html#0-7-2-2019-02-25","title":"0.7.2 (2019-02-25) - Changelog","type":"extras"},{"doc":"* Patches as per Last.fm API JSON data format changes: uts timestamp, play counts info are now returned as integers instead of strings.","ref":"changelog.html#0-7-1-2019-01-14","title":"0.7.1 (2019-01-14) - Changelog","type":"extras"},{"doc":"* `sync/0`, `sync/1`: sync and keep tracks of scrobbles for a default and Last.fm users, via delta archiving (download latest scrobbles)","ref":"changelog.html#0-7-0-2019-01-11","title":"0.7.0 (2019-01-11) - Changelog","type":"extras"},{"doc":"* Support for Solr: load all transformed (TSV) data from the archive into Solr, `load_archive/2`\n* Underpinning functions to read, parse, load data into Solr","ref":"changelog.html#0-6-0-2018-11-10","title":"0.6.0 (2018-11-10) - Changelog","type":"extras"},{"doc":"* `transform_archive/2`: transform downloaded raw Last.fm archive and create a TSV file archive\n*  Underpinning functions to read, parse and transform raw Lastfm JSON data into TSV files","ref":"changelog.html#0-5-0-2018-11-05","title":"0.5.0 (2018-11-05) - Changelog","type":"extras"},{"doc":"* fix single year archiving (bug): `daily: true` option","ref":"changelog.html#0-4-1-2018-11-01","title":"0.4.1 (2018-11-01) - Changelog","type":"extras"},{"doc":"* `archive/3`: archiving data subset based on date ranges: single day/year, past week/month, arbitrary date range using `Date`, `Date.Range` structs\n* `daily: true` option for finer-grained batch archiving cf. the default year-level granularity","ref":"changelog.html#0-4-0-2018-10-31","title":"0.4.0 (2018-10-31) - Changelog","type":"extras"},{"doc":"* `overwrite` archiving option to also re-fetch any existing downloaded data, for refreshing file archive","ref":"changelog.html#0-3-2-2018-10-27","title":"0.3.2 (2018-10-27) - Changelog","type":"extras"},{"doc":"* Keyword list archiving options (`per_page`, `interval`) for `archive/2` which can also be configured","ref":"changelog.html#0-3-1-2018-10-27","title":"0.3.1 (2018-10-27) - Changelog","type":"extras"},{"doc":"* `archive` latest tracks (current year) on a daily basis to better ensure data immutability and updatability (new scrobbles)\n* `archive` older tracks on a yearly basis","ref":"changelog.html#0-3-0-2018-10-26","title":"0.3.0 (2018-10-26) - Changelog","type":"extras"},{"doc":"* `archive/0`: downloads scrobbled tracks, creates a file archive for a default user according to configuration settings\n* `archive/2`: downloads scrobbled tracks, creates a file archive for any Lastfm user\n* `write/3`: outputs data for multiple Lastfm users (no longer hardwired to the default user)","ref":"changelog.html#0-2-0-2018-10-23","title":"0.2.0 (2018-10-23) - Changelog","type":"extras"},{"doc":"* Download scrobbled tracks raw data, create an archive on local filesystem for a default user in configuration - `archive/1`\n* `extract/5` and `write/2` functions for Lastfm API requests and file output","ref":"changelog.html#0-1-0-2018-10-22","title":"0.1.0 (2018-10-22) - Changelog","type":"extras"},{"doc":"# Setup, installation\n\n```elixir\nMix.install(\n  [{:lastfm_archive, \"~> 1.2\"}],\n  config: [\n    lastfm_archive: [\n      data_dir: \"./lastfm_data/\",\n      user: \"\"\n    ]\n  ]\n)\n\n:ok\n```","ref":"setup.html","title":"Setup, installation","type":"extras"},{"doc":"* [install](https://livebook.dev/#install) and start Livebook\n* run this Livebook guide\n* configure Livebook as instructued below, click on `Notebook dependencies and setup`, `Setup` (above)","ref":"setup.html#installation","title":"Installation - Setup, installation","type":"extras"},{"doc":"Lastfm archive has been configured as a dependency in `Setup` above. You need to check and modify the following configs:\n\n1. **`user`**: specify a Lastfm username in this config or share it as `LFM_USER` secret (see [Secrets management](https://hackmd.io/@hugobarauna/By6ulTnMi#Secret-management))\n2. **`data_dir`** (optional): by default archive data is stored in `~/lastfm_data/` directory within your home directory. Modifiy this location if other directory is preferred\n\n \n\n#","ref":"setup.html#configuration","title":"Configuration - Setup, installation","type":"extras"},{"doc":"To access the API provided by Lastfm, you need to [apply for a 32-digit string key](https://www.last.fm/api/authentication). This key will be used when downloading data from the API.\n\nAdd the key via a `LFM_API_KEY` secret and share it with this Livebook - see [Secrets management](https://hackmd.io/@hugobarauna/By6ulTnMi#Secret-management).","ref":"setup.html#api-key-environment-variable","title":"API key environment variable - Setup, installation","type":"extras"},{"doc":"Run the following code (click `Evaluate`) to send a test request to Lastfm. If you receive basic details from Lastfm, then you're all set.\n\n```elixir\nLastfmArchive.Livebook.info()\n```","ref":"setup.html#test-the-setup","title":"Test the setup - Setup, installation","type":"extras"},{"doc":"# Creating a file archive\n\n```elixir\nMix.install(\n  [{:lastfm_archive, \"~> 1.2\"}],\n  config: [\n    lastfm_archive: [\n      data_dir: \"./lastfm_data/\",\n      user: \"\"\n    ]\n  ]\n)\n\nalias LastfmArchive.Livebook, as: LFM_LB\n:ok\n```","ref":"archiving.html","title":"Creating a file archive","type":"extras"},{"doc":"This is a step-by-step guide to creating a local file archive containing [Last.fm scrobbles](https://www.last.fm/about/trackmymusic), i.e. music tracks that you have been listening to. This uses the [lastfm_archive](https://github.com/boonious/lastfm_archive) tool that facilicates scrobbles downloading from the Last.fm API and stores them in raw JSON files.\n\n**Why?** This grounds your data, backs up your music listening history, keeping it safe. You can also use the archive personally in many ways, in other applications. For example, [transforming the data](https://hexdocs.pm/lastfm_archive/transforming.html) into suitable formats for interesting analytics and visualisation.\n\n#","ref":"archiving.html#introduction","title":"Introduction - Creating a file archive","type":"extras"},{"doc":"* [Setup, installation](https://hexdocs.pm/lastfm_archive/setup.html) Livebook","ref":"archiving.html#prerequisite","title":"Prerequisite - Creating a file archive","type":"extras"},{"doc":"Run the following code (click `Evaluate` below) to find out what are you about to archive:\n\n```elixir\nLFM_LB.info()\n```","ref":"archiving.html#scope","title":"Scope - Creating a file archive","type":"extras"},{"doc":"Run the follow to begin archiving. This will initate a process that fetches daily scrobbles at around every 1s (within permissable Lastfm API request rate). The process is memoised or cached. It can be halted (click `Stop`) and resumed without data being re-fetched. This prevents unnecessarily calls being made to the API.\n\n```elixir\nLastfmArchive.sync()\n```","ref":"archiving.html#archiving","title":"Archiving - Creating a file archive","type":"extras"},{"doc":"Run the following to visualise the archiving process and progress. The code displays daily and annual playcounts stats. It also provides heatmaps visualisation. Rerun the code to get the latest status.\n\nSee a [sample output](https://hexdocs.pm/lastfm_archive/assets/img/livebook_heatmap.png).\n\n```elixir\nLastfmArchive.default_user() |> LFM_LB.render_playcounts_heatmaps()\n```","ref":"archiving.html#status-daily-playcounts","title":"Status: daily playcounts - Creating a file archive","type":"extras"},{"doc":"`:year`: archiving scrobbles from a given year option. For example:\n\n```elixir\n# change year to one containing scrobbles\nLastfmArchive.default_user() |> LastfmArchive.sync(year: 2023)\n```","ref":"archiving.html#year-option","title":"Year option - Creating a file archive","type":"extras"},{"doc":"`:date` archiving scrobbles from a given date\n\n```elixir\n# change date to one containing scrobbles\nLastfmArchive.default_user() |> LastfmArchive.sync(date: ~D[2023-09-03])\n```","ref":"archiving.html#date-option","title":"Date option - Creating a file archive","type":"extras"},{"doc":"`:overwrite` any existing scrobbles, ignoring archiving status cache\n\n```elixir\nLastfmArchive.default_user() |> LastfmArchive.sync(date: ~D[2023-09-03], overwrite: true)\n```","ref":"archiving.html#overwrite-option","title":"Overwrite option - Creating a file archive","type":"extras"},{"doc":"# Columnar data transforms\n\n```elixir\nMix.install(\n  [{:lastfm_archive, \"~> 1.2\"}, {:kino_explorer, \"~> 0.1.8\"}],\n  config: [\n    lastfm_archive: [\n      data_dir: \"./lastfm_data/\",\n      user: \"\"\n    ]\n  ]\n)\n\n:ok\n```","ref":"transforming.html","title":"Columnar data transforms","type":"extras"},{"doc":"This guide uses [lastfm_archive](https://github.com/boonious/lastfm_archive)\nto create various columnar data archives, enabling an entire dataset of scrobbles for a Lastfm user to be read into a data frame for analytics purposes.\n\n#","ref":"transforming.html#introduction","title":"Introduction - Columnar data transforms","type":"extras"},{"doc":"* [Setup, installation](https://hexdocs.pm/lastfm_archive/setup.html)\n* [Creating a file archive](https://hexdocs.pm/lastfm_archive/archiving.html) containing scrobbles in raw JSON format fetched from Lastfm API","ref":"transforming.html#prerequisite","title":"Prerequisite - Columnar data transforms","type":"extras"},{"doc":"The default file archive consists of data downloaded from Lastfm that is stored in per-day raw data format (a JSON file per day). It is not optimised for analytics and computational purposes. For example, all the raw data files must be read, parsed, analysed and consolidated, even for a simple metric such as counting the total number of albums scrobbled. For example, if you have 16 years of scrobbles, there are a lot of files to read just to get a simple count!\n\nColumnar based storage is [better for analytics](https://thenewstack.io/what-is-a-column-database-and-when-should-you-use-one/#:~:text=The%20main%20advantage%20of%20a,than%20traditional%20row%2Dbased%20databases.), [OLAP](https://www.influxdata.com/glossary/olap/?utm_source=vendor&utm_medium=referral&utm_campaign=2022-12_spnsr-ctn_column-database_tns&utm_content=inline-mention) workloads and for historical archive. [lastfm_archive](https://github.com/boonious/lastfm_archive) provides capability to transform the raw JSON archive into the following storage formats:\n\n* [Apache Arrow](https://arrow.apache.org) columnar format\n* [Apache Parquet](https://parquet.apache.org) columnar format\n* also CSV (tab-delimited)","ref":"transforming.html#transform-to-columnar-formats","title":"Transform to columnar formats - Columnar data transforms","type":"extras"},{"doc":"Run the following code to transform the file archive into an [Apache Parquet](https://parquet.apache.org/docs/overview/) archive.\n\n```elixir\nuser = LastfmArchive.default_user()\nLastfmArchive.transform(user, format: :parquet)\n```\n\nTo transform / regenerate a single year, use the `overwrite` (old data) and `year` options, below assumes the file archive contains scrobbles from year 2023 (otherwise, please experiment with other years):\n\n```elixir\nLastfmArchive.transform(user, format: :parquet, overwrite: true, year: 2023)\n```\n\nTo simply transform / regenerate the entire archive, overwriting all previous data:\n\n```elixir\nLastfmArchive.transform(user, format: :parquet, overwrite: true)\n```","ref":"transforming.html#apache-parquet-archive","title":"Apache Parquet archive - Columnar data transforms","type":"extras"},{"doc":"[Apache Arrow](https://arrow.apache.org/overview) is an in-memory columnar format that is interoperable among data applications written in different languages. Arrow data is serialised according to an interprocess communication (`IPC`) protocol.\n\nRun the following code to create an Apache Arrow archive according its IPC streaming format:\n\n```elixir\nLastfmArchive.default_user() |> LastfmArchive.transform(format: :ipc_stream)\n```\n\nThe same `overwrite` and `year` options are applicable (see [Apache Parquet archive](#apache-parquet-archive)) for regenerating / transforming all or single-year data.","ref":"transforming.html#apache-arrow-archive","title":"Apache Arrow archive - Columnar data transforms","type":"extras"},{"doc":"Columnar data can be read into an [Explorer data frame](https://hexdocs.pm/explorer/Explorer.DataFrame.html) for analysis. To read a single-year, single-column scrobbles data from the Arrow IPC archive into a data frame, run (again assuming year 2023 scrobbles, otherwise try another `year`):\n\n```elixir\nuser = LastfmArchive.default_user()\n{:ok, df} = LastfmArchive.read(user, format: :ipc_stream, year: 2023)\n```\n\nThe data frame can now be used for various analytics workloads. For example, compute all unique albums scrobbled in year 2023 and list them in descending order (most scrobbled albums):\n\n```elixir\ndf |> Explorer.DataFrame.collect() |> Explorer.DataFrame.frequencies([:album])\n```\n\nTo read the **entire dataset** into a data frame, run:\n\n```elixir\n{:ok, df_all} = LastfmArchive.read(user, format: :ipc_stream)\n```\n\nAnd use the data frame for various analytics, for example compute all unique artists, run:\n\n```elixir\ndf_all |> Explorer.DataFrame.collect() |> Explorer.DataFrame.frequencies([:artist])\n```\n\nTo compute all unique tracks by artists:\n\n```elixir\ndf_all |> Explorer.DataFrame.collect() |> Explorer.DataFrame.frequencies([:track, :artist])\n```","ref":"transforming.html#read-columnar-data-for-analytics","title":"Read columnar data for analytics - Columnar data transforms","type":"extras"},{"doc":"# Facets archiving\n\n```elixir\nMix.install(\n  [{:lastfm_archive, \"~> 1.2\"}, {:kino_explorer, \"~> 0.1.10\"}],\n  config: [\n    lastfm_archive: [\n      data_dir: \"./lastfm_data/\",\n      user: \"\"\n    ]\n  ]\n)\n\nalias Explorer.DataFrame\nalias LastfmArchive.Livebook, as: LFM_LB\n\n:ok\n```","ref":"facets.html","title":"Facets archiving","type":"extras"},{"doc":"`lastfm_archive` data stemmed from music tracks have been played over times and some, repeatedly. The data has multiple facets (aspects or dimensions). For example, what are the unique artists, albums or tracks within the listening history? When was a particular track played for the very first time or recently? Such faceted data may be derived from the scrobbles, typically from a column or columns subset of the archive.\n\nUsually, facets data can be computated as required in runtime. For example, when a track was scrobbled, extra info can be used to check if it's new, popular or hasn't been played for awhile. This requires the entire listening history to be analysed *in situ*. For a larger dataset (e.g. 16-year listening history) this may be slow and computationally expensive.\n\nThis guide demonstrates how [lastfm_archive](https://github.com/boonious/lastfm_archive)\ncan be used to derive the following types of archives which are essentially **pre-created facet datasets**. It also exemplifies the data usage in analytics and visualisation.\n\n* `artists`\n* `albums`\n* `tracks`\n\n#","ref":"facets.html#introduction","title":"Introduction - Facets archiving","type":"extras"},{"doc":"* [Setup, installation](https://hexdocs.pm/lastfm_archive/setup.html)\n* [Creating a file archive](https://hexdocs.pm/lastfm_archive/archiving.html) containing scrobbles in raw JSON format fetched from Lastfm API","ref":"facets.html#prerequisite","title":"Prerequisite - Facets archiving","type":"extras"},{"doc":"Using [`read/2`](https://hexdocs.pm/lastfm_archive/LastfmArchive.html#read/2) that returns a lazy dataframe of the entire listening history, a faceted dataframe can be created via the following pipeline. Aggregation is performed on a facet, e.g. the `artist` column, deriving all unique values and additional stats such as `first_play` - the earliest occurrence of an artist. Evaluate the code below to see a dataframe consisting all the artists to whom you have listened.\n\nThis pipeline is used in [`FacetsTransformer`](https://github.com/boonious/lastfm_archive/blob/master/lib/lastfm_archive/archive/transformers/facets_transformer.ex) which is part of the lastfm_archive [`transform/2`](https://hexdocs.pm/lastfm_archive/LastfmArchive.html#transform/2) function.\n\n```elixir\nrequire DataFrame\n{:ok, df} = LastfmArchive.default_user() |> LastfmArchive.read(format: :ipc_stream)\n\ndf\n|> DataFrame.select([:artist, :datetime])\n|> DataFrame.group_by([:artist])\n|> DataFrame.summarise(counts: count(datetime), first_play: min(datetime))\n|> DataFrame.arrange(desc: counts)\n|> DataFrame.collect()\n```","ref":"facets.html#creating-a-facet-dataframe","title":"Creating a facet dataframe - Facets archiving","type":"extras"},{"doc":"`lastfm_archive`'s transform function contains the above logic. It enables the creation of facet datasets using the following main options:\n\n* facet: `:artists`, `:albums` or `:tracks`\n* format: `:ipc_stream`, `:ipc`, `:parquet` or `:csv`\n\nThe archive is stored on a year-by-year basis so that subsequent data updates may be done without the need to re-genereate all previous data. Only the latest data (year) needs to be refreshed (via the `year` option).\n\n```elixir\nLastfmArchive.default_user() |> LastfmArchive.transform(facet: :artists, format: :ipc_stream)\n```","ref":"facets.html#facets-archiving","title":"Facets archiving - Facets archiving","type":"extras"},{"doc":"The archives can be read and used in many ways. For example, the following identifies new artists played for the very first time on a particular date.\n\nSee a [sample output](https://hexdocs.pm/lastfm_archive/assets/img/livebook_new_artists_on_this_day.png).\n\n```elixir\nrequire DataFrame\n# change facet to `:albums` or `:tracks` to see other results\n{:ok, df} = LastfmArchive.read(LastfmArchive.default_user(), facet: :artists, format: :ipc_stream)\n\n# change these dates to ones that will yield results for your dataset\nthis_day_am = DateTime.new!(~D[2022-09-02], ~T[00:00:00], \"Etc/UTC\") |> DateTime.to_naive()\nthis_day_pm = DateTime.new!(~D[2022-09-02], ~T[23:59:59], \"Etc/UTC\") |> DateTime.to_naive()\n\ndf\n|> DataFrame.filter(first_play > ^this_day_am and first_play   DataFrame.arrange(desc: counts)\n|> DataFrame.head()\n|> DataFrame.collect()\n```\n\nYou can also use the data in visualisation. For example, the following displays all artists and the time when their were discovered / first listened to, in a [VegaLite](https://vega.github.io/vega-lite/) bubble plot. Artists are represented by bubbles of different sizes and colours proportional to the overall total counts (popularity).\n\nSee a [sample output](https://hexdocs.pm/lastfm_archive/assets/img/livebook_firstplay_bubble_plot.png) generated from a Lastfm user's listening history.\n\nThe plot shows regular discovery of new artists. Artists discovered earlier are more \"popular\", i.e. more likelihood of repeats over time. You can see the [source code of the plot here](https://github.com/boonious/lastfm_archive/blob/master/lib/lastfm_archive/livebook.ex), look for the `render_first_play_bubble_plot` function.\n\n```elixir\ndf |> DataFrame.collect() |> LFM_LB.render_first_play_bubble_plot()\n```","ref":"facets.html#using-faceted-archive","title":"Using faceted archive - Facets archiving","type":"extras"},{"doc":"The other `transform/2` options may also be used for example, to overwrite existing year 2023 data (below) after new scrobbles have been synced from Lastfm. You can also regenerate the entire dataset with `overwrite` option without the `year` option.\n\n```elixir\nLastfmArchive.default_user()\n|> LastfmArchive.transform(facet: :artists, format: :ipc_stream, year: 2023, overwrite: true)\n```","ref":"facets.html#other-transform-options","title":"Other transform options - Facets archiving","type":"extras"}]}